"""
A demo of creating a new database via SQL Alchemy.
Under MIT License from sprin (https://gist.github.com/sprin/5846464/)
This module takes the form of a nosetest with three steps:
    - Set up the new database.
    - Create a table in the new database.
    - Teardown the new database.
"""
from sqlalchemy import (
    create_engine,
    MetaData,
    Table,
    Column,
    Integer,
)
from sqlalchemy.pool import NullPool


# XXX: It is advised to use another user that can connect to a default database,
# and has CREATE DATABASE permissions, rather than use a superuser.
from bullhorn_interface.settings.settings import DB_USER, DB_PASSWORD

DB_CONFIG_DICT = {
        'user': DB_USER,
        'password': DB_PASSWORD,
        'host': 'localhost',
        'port': 5432,
}

DB_CONN_FORMAT = "postgresql://{user}:{password}@{host}:{port}/{database}"

DB_CONN_URI_DEFAULT = (DB_CONN_FORMAT.format(
    database='postgres',
    **DB_CONFIG_DICT))

engine_default = create_engine(DB_CONN_URI_DEFAULT)

NEW_DB_NAME = 'test'

DB_CONN_URI_NEW = (DB_CONN_FORMAT.format(
    database=NEW_DB_NAME,
    **DB_CONFIG_DICT))

metadata = MetaData()

proj = Table('test', metadata, Column('id', Integer))

def setup_module(db_name=NEW_DB_NAME):
    conn = engine_default.connect()
    conn.execute("COMMIT")
    # Do not substitute user-supplied database names here.
    conn.execute("CREATE DATABASE %s" % db_name)
    conn.close()

def test_create_table():
    # Get a new engine for the just-created database and create a table.
    engine_new = create_engine(DB_CONN_URI_NEW, poolclass=NullPool)
    conn = engine_new.connect()
    metadata.create_all(conn)
    conn.close()

def teardown_module():
    conn = engine_default.connect()
    conn.execute("COMMIT")
    # Do not substitute user-supplied database names here.
    conn.execute("DROP DATABASE %s" % NEW_DB_NAME)
    conn.close()




import json
import os

from sqlalchemy import (
    create_engine,
    MetaData,
    Table,
    Column,
    Integer,
    String
)
from sqlalchemy.pool import NullPool
from bullhorn_interface.settings import settings


DB_CONFIG_DICT = {
        'user': settings.DB_USER,
        'password': settings.DB_PASSWORD,
        'host': 'localhost',
        'port': 5432,
}
DB_CONN_FORMAT = "postgresql://{user}:{password}@{host}:{port}/{database}"
DB_CONN_URI_DEFAULT = (DB_CONN_FORMAT.format(
    database='postgres',
    **DB_CONFIG_DICT))
engine_default = create_engine(DB_CONN_URI_DEFAULT)
NEW_DB_NAME = 'bullhorn'
DB_CONN_URI_NEW = (DB_CONN_FORMAT.format(
    database=NEW_DB_NAME,
    **DB_CONFIG_DICT))
metadata = MetaData()
class LoginToken:
    table_name = 'login_token'
    table_name_id = table_name + '_pk'
    table = Table(table_name, metadata,
        Column(table_name_id, Integer, primary_key=True),
        Column('access_token', String(45), nullable=False),
        Column('expires_in', Integer, nullable=False),
        Column('refresh_token', String(45), nullable=False),
        Column('token_type', String(45), nullable=False),
        Column('expiry', Integer, nullable=False),
    )
class AccessToken:
    table_name = 'access_token'
    table_name_id = table_name + '_pk'
    table = Table(table_name, metadata,
        Column(table_name_id, Integer, primary_key=True),
        Column('bh_rest_token', String(45), nullable=False),
        Column('rest_url', String(60), nullable=False)
    )


def select_token(table_name):
    if settings.USE_FLAT_FILES:
        print(settings.USE_FLAT_FILES)
        with open(os.path.join(settings.SETTINGS_DIR, f'{table_name}.json')) as token_file:
            token = json.load(token_file)
            token_file.close()
            return [token]
    else:
        engine = create_engine(DB_CONN_URI_NEW, poolclass=NullPool)
        data = MetaData(engine)
        table = Table(table_name, data, autoload=True)
        conn = engine.connect()
        result = table.select().order_by(table_name + "_pk desc").execute()
        conn.close()
        return result

def insert_token(table, kwargs):
    if settings.USE_FLAT_FILES:
        with open(os.path.join(settings.SETTINGS_DIR, f'{table}.json'), 'w') as token_file:
            json.dump(kwargs, token_file, indent=2, sort_keys=True)
            token_file.close()
    else:
        engine = create_engine(DB_CONN_URI_NEW, poolclass=NullPool)
        data = MetaData(engine)
        table = Table(table, data, autoload=True)
        conn = engine.connect()
        i = table.insert()
        i.execute(**kwargs)
        conn.close()


def setup_module(db_name=NEW_DB_NAME):
    conn = engine_default.connect()
    conn.execute("COMMIT")
    # Do not substitute user-supplied database names here.
    conn.execute("CREATE DATABASE %s" % db_name)
    conn.close()

def create_table():
    # Get a new engine for the just-created database and create a table.
    engine_new = create_engine(DB_CONN_URI_NEW, poolclass=NullPool)
    conn = engine_new.connect()
    metadata.create_all(conn)
    conn.close()


def teardown_module(db_name=NEW_DB_NAME):
    conn = engine_default.connect()
    conn.execute("COMMIT")
    # Do not substitute user-supplied database names here.
    conn.execute("DROP DATABASE %s" % db_name)
    conn.close()


import datetime
import json

import requests

from bullhorn_interface.alchemy.bullhorn_db import insert_token, select_token
from bullhorn_interface.settings.settings import CLIENT_ID, CLIENT_SECRET


def login(grant_type="authorization_code", client_id=CLIENT_ID, client_secret=CLIENT_SECRET, code=""):
    browser_url = f"https://auth.bullhornstaffing.com/oauth/authorize?client_id={CLIENT_ID}&response_type=code"
    example_redirect_url = ["http://www.bullhorn.com/?code=",
                            "{YOUR CODE WILL BE RIGHT HERE}",
                            f"&client_id={CLIENT_ID}"]

    if not code:
        print(f"Paste this URL into browser {browser_url}. \n"
              f"Redirect URL will look like this: {''.join(example_redirect_url)}.\n")
    else:
        url = "https://auth.bullhornstaffing.com/oauth/token?grant_type=authorization_code"
        url = url + f"&client_secret={client_secret}&client_id={client_id}&code={code}"
        response = requests.post(url)
        login_token = json.loads(response.text)
        login_token['expiry'] = datetime.datetime.now().timestamp() + login_token["expires_in"]
        insert_token('login_token', login_token, )
        print(f"New Access Token: {login_token['access_token']}")

def refresh_token():

    for row in select_token('login_token'):
        tokens = row
        break

    url = "https://auth.bullhornstaffing.com/oauth/token?grant_type=refresh_token"
    url = url + f"&refresh_token={tokens['refresh_token']}&client_id={CLIENT_ID}"
    url = url + f"&client_secret={CLIENT_SECRET}"
    response = requests.post(url)
    login_token = json.loads(response.text)
    login_token['expiry'] = datetime.datetime.now().timestamp() + login_token["expires_in"]
    insert_token('login_token', login_token, )
    return f"New Access Token: {login_token['access_token']}"

def get_api_token():

    for row in select_token('login_token'):
        login_token = row
        break

    url = f"https://rest.bullhornstaffing.com/rest-services/login?version=*&access_token={login_token['access_token']}"
    response = requests.get(url)
    temp_token = json.loads(response.text)
    access_token = {"bh_rest_token": temp_token["BhRestToken"], "rest_url": temp_token["restUrl"]}
    insert_token('access_token', access_token, )
    return json.dumps(access_token, indent=2, sort_keys=True)


def api_call(command="search", method="", entity="", entity_id="",
             select_fields=[], query="",
             auto_refresh=True, body="", kwargs={}):

    if command == "search" or command == "query":
        # defaults for easy testing
        if not entity:
            entity = "Candidate"
        if not entity_id and not query:
            query = "id:1"
        if not select_fields:
            select_fields = ["id", "firstName", "middleName", "lastName", "comments", "notes(*)"]
        request_func = requests.get

    elif command == "entity":
        if method == "UPDATE":
            request_func = requests.post
        elif method == "CREATE":
            request_func = requests.put
        elif method == "GET":
            request_func = requests.get

    if auto_refresh:
        print('Refreshing Access Tokens')
        refresh_token()
        get_api_token()

    for row in select_token('access_token'):
        access_token = row
        break

    rest_url = access_token['rest_url']
    rest_token = access_token['bh_rest_token']

    entity_id_str = f"/{entity_id}" if entity_id else ""
    url = f"{rest_url}/{command}/{entity}{entity_id_str}?BhRestToken={rest_token}"

    if select_fields:
        if type(select_fields) is str:
            url += f"&fields={select_fields}"
        elif type(select_fields) is list:
            url += f"&fields={','.join(select_fields)}"
        else:
            raise TypeError('select_fields must be a str or list object.')

    if query:
        url += f"&query={query}"

    for key in kwargs.keys():
        url += f"&{key}={kwargs[key]}"

    response = request_func(url, json=body)

    # print(url)
    # print(response.text)

    return json.loads(response.text)


import json
import os

from bullhorn_interface.helpers import __except__
from bullhorn_interface.tests import ImproperlyConfigured

SETTINGS_DIR = os.path.dirname(os.path.abspath(__file__))
PROJECT_DIR = os.path.dirname(SETTINGS_DIR)


def create_conf():
    conf_dict = {
        "USE_FLAT_FILES": True,
        "SECRETS_LOCATION": "bullhorn_secrets.py"
    }
    with open(os.path.join(SETTINGS_DIR, 'conf.py'), 'w') as conf:
        conf.write(json.dumps(conf_dict, indent=4))
    return load_conf()


def create_secrets():
    secrets_dict = {
          "CLIENT_ID": "",
          "CLIENT_SECRET": "",
          "EMAIL_ADDRESS": "",
          "EMAIL_PASSWORD": "",
          "DB_USER": "",
          "DB_PASSWORD": ""
    }
    with open(os.path.join(SETTINGS_DIR, 'bullhorn_secrets.py'), 'w') as secrets:
        secrets.write(json.dumps(secrets_dict, indent=4))
    return load_secrets()


@__except__(FileNotFoundError, lambda: create_conf())
def load_conf():

    with open(os.path.join(SETTINGS_DIR, 'conf.py')) as conf:
        conf = json.load(conf)
    return conf


@__except__(FileNotFoundError, lambda: create_secrets())
def load_secrets():
    conf = load_conf()
    with open(conf['SECRETS_LOCATION']) as secrets:
        secrets = json.load(secrets)

    try:
        USE_FLAT_FILES = conf["USE_FLAT_FILES"]
        CLIENT_ID = secrets["CLIENT_ID"]
        CLIENT_SECRET = secrets["CLIENT_SECRET"]
        EMAIL_ADDRESS = secrets["EMAIL_ADDRESS"]
        EMAIL_PASSWORD = secrets["EMAIL_PASSWORD"]
        DB_USER = secrets["DB_USER"]
        DB_PASSWORD = secrets["DB_PASSWORD"]
    except KeyError as e:
        if "USE_FLAT_FILES" in e.args[0]:
            raise ImproperlyConfigured(f'{e.args[0]} not found in conf.py')
        else:
            raise ImproperlyConfigured(f'{e.args[0]} not found in {conf["SECRETS_LOCATION"]}')
    return secrets

secrets, conf = load_secrets(), load_conf()

USE_FLAT_FILES = conf["USE_FLAT_FILES"]
CLIENT_ID = secrets["CLIENT_ID"]
CLIENT_SECRET = secrets["CLIENT_SECRET"]
EMAIL_ADDRESS = secrets["EMAIL_ADDRESS"]
EMAIL_PASSWORD = secrets["EMAIL_PASSWORD"]
DB_USER = secrets["DB_USER"]
DB_PASSWORD = secrets["DB_PASSWORD"]


import json
import os

from bullhorn_interface.helpers import __except__


class ImproperlyConfigured(BaseException):
    pass

def missing_conf():
    return ImproperlyConfigured('\n\nMissing conf.py or bullhorn_secrets.py. \n'
                               'Make sure that your conf.py file exists in '
                               '/bullhorn_interface/settings/ and designates a "SECRETS_LOCATION".')


def missing_conf_location():
    return ImproperlyConfigured('SECRETS_LOCATION not found in conf.py')


@__except__(FileNotFoundError, lambda: (_ for _ in ()).throw(missing_conf()))
@__except__(KeyError, lambda: (_ for _ in ()).throw(missing_conf_location()))
def conf_exists_test():
    from bullhorn_interface.settings.settings import SETTINGS_DIR
    with open(os.path.join(SETTINGS_DIR, 'conf.py')) as conf:
        conf = json.load(conf)
        with open(os.path.join(SETTINGS_DIR, conf['SECRETS_LOCATION'])) as secrets:
            return json.load(secrets), conf


def valid_conf_test():
    secrets, conf = conf_exists_test()
    try:
            USE_FLAT_FILES = conf["USE_FLAT_FILES"]
            CLIENT_ID = secrets["CLIENT_ID"]
            CLIENT_SECRET = secrets["CLIENT_SECRET"]
            EMAIL_ADDRESS = secrets["EMAIL_ADDRESS"]
            EMAIL_PASSWORD = secrets["EMAIL_PASSWORD"]
            DB_USER = secrets["DB_USER"]
            DB_PASSWORD = secrets["DB_PASSWORD"]
    except KeyError as e:
        if "USE_FLAT_FILES" in e.args[0]:
            raise ImproperlyConfigured(f'{e.args[0]} not found in conf.py')
        else:
            raise ImproperlyConfigured(f'{e.args[0]} not found in {conf["SECRETS_LOCATION"]}')
    print('Test Passed.')


import getpass
import json
import os
import time
from math import trunc


# from bullhorn.settings.settings import EMAIL_ADDRESS, EMAIL_PASSWORD, SETTINGS_DIR

# from settings.settings import EMAIL_ADDRESS, EMAIL_PASSWORD, PROJECT_DIR


def send_email(recipient, subject, body, user=None, pwd=None):

    import smtplib
    from bullhorn_interface.settings.settings import EMAIL_ADDRESS, EMAIL_PASSWORD

    user = user if user else EMAIL_ADDRESS
    pwd = pwd if pwd else EMAIL_PASSWORD

    gmail_user = user
    gmail_pwd = pwd
    FROM = user
    TO = recipient if type(recipient) is list else [recipient]
    SUBJECT = subject
    TEXT = body

    # Prepare actual message
    message = """From: %s\nTo: %s\nSubject: %s\n\n%s
    """ % (FROM, ", ".join(TO), SUBJECT, TEXT)
    try:
        server = smtplib.SMTP("smtp.gmail.com", 587)
        server.ehlo()
        server.starttls()
        server.login(gmail_user, gmail_pwd)
        server.sendmail(FROM, TO, message)
        server.close()
        print('Email Sent.')
    except:
        print("failed to send mail")


def time_check(start):
    current_time = time.time()
    auto_refresh = False
    hours, rem = divmod(current_time - start, 3600)
    minutes, seconds = divmod(rem, 60)
    if minutes > 7:
        start = time.time()
        auto_refresh = True

    return start, auto_refresh


def print_duration(start):
    current_time = time.time()
    hours, rem = divmod(current_time - start, 3600)
    minutes, seconds = divmod(rem, 60)
    time_string = f'{trunc(hours):02d}:{trunc(minutes):02d}:{trunc(seconds):02d}'
    return f'{time_string:>12}'


def requirements():
    from bullhorn_interface.settings.settings import PROJECT_DIR
    with open(os.path.join(os.path.dirname(PROJECT_DIR), 'requirements.txt')) as requirements_file:
        req = map(lambda x: x.replace('\n', ''), requirements_file.readlines())
        return list(req)


def set_secrets():
    from bullhorn_interface.settings.settings import SETTINGS_DIR
    create, designate = False, False
    create_or_designate = str(input('Would you like to: \n'
                   '\t1: Create a new file named bullhorn_secrets.py and store it in a specified path?\n'
                   '\t2: Specify the full path of an existing secrets file?\n'))
    if '1' in create_or_designate:
        create = True
    elif '2' in create_or_designate:
        designate = True
    else:
        print('Please specify option 1 or 2 by typing 1 or 2.')
        return

    if create:
        SECRETS_LOCATION = input("1 selected. Please specify the full path containing your secrets file: "
                                      "(/path/containing/secrets/)").replace("\n", "")
    else:
        SECRETS_LOCATION = input("2 selected. Please specify the name of your secrets file "
                                      "(/path/to/secrets.json): ").replace("\n", "")

    if create:
        with open(os.path.join(SETTINGS_DIR, 'conf.py')) as conf:
            USE_FLAT_FILES = json.load(conf)['USE_FLAT_FILES']
        SECRETS_LOCATION = os.path.join(SECRETS_LOCATION, 'bullhorn_secrets.json')
        secrets = {
            "CLIENT_ID": input("Please input your Bullhorn Client ID for API development: "),
            "CLIENT_SECRET": getpass.getpass("Bullhorn Client Secret: "),
            "EMAIL_ADDRESS": input("Default gmail address for Bullhorn API Interface used in helpers.send_mail(): "),
            "EMAIL_PASSWORD": getpass.getpass("Default gmail passwrd for Bullhorn API Interface used in helpers.send_mail(): "),
            "DB_USER": input("PostgreSQL database login role username. (Database used to store access and API tokens): "),
            "DB_PASSWORD": getpass.getpass("PostgreSQL database login role password. (Database used to store access and API tokens): "),
        }
        with open(os.path.join(SETTINGS_DIR, 'conf.py'), 'w') as new_conf:
            new_conf_dict = {"USE_FLAT_FILES": USE_FLAT_FILES, "SECRETS_LOCATION": SECRETS_LOCATION}
            new_conf.write(json.dumps(new_conf_dict, indent=4))
        with open(os.path.join(SECRETS_LOCATION), 'w') as new_secrets:
            new_secrets.write(json.dumps(secrets, indent=4))

    elif designate:
        with open(os.path.join(SETTINGS_DIR, 'conf.py')) as conf:
            USE_FLAT_FILES = json.load(conf)['USE_FLAT_FILES']
        with open(os.path.join(SETTINGS_DIR, 'conf.py'), 'w') as new_conf:
            new_conf_dict = {"USE_FLAT_FILES": USE_FLAT_FILES, "SECRETS_LOCATION": SECRETS_LOCATION}
            new_conf.write(json.dumps(new_conf_dict, indent=4))
    refresh_settings()


def set_conf():
    from bullhorn_interface.settings.settings import SETTINGS_DIR
    use_flat_or_not = input("Would you like to store your access tokens and login tokens in flat files \n"
                            "or in a postgreSQL database? \n"
                            "\t1: PostgreSQL Database\n"
                            "\t2: Flat Files (Default)\n\n"
                            "Note: Flat files may experience concurrency problems when making simultaneous"
                            " API calls.\n")

    if '1' in use_flat_or_not:
        flat = False
    else:
        flat = True

    print(f'{2 if flat else 1} selected.')

    with open(os.path.join(SETTINGS_DIR, 'conf.py')) as conf:
        SECRETS_LOCATION = json.load(conf).get('SECRETS_LOCATION')
    with open(os.path.join(SETTINGS_DIR, 'conf.py'), 'w') as new_conf:
        new_conf_dict = {"USE_FLAT_FILES": flat, "SECRETS_LOCATION": SECRETS_LOCATION}
        new_conf.write(json.dumps(new_conf_dict, indent=4))
    refresh_settings()


def refresh_settings():
    import bullhorn_interface.settings.settings as settings
    from bullhorn_interface.tests import ImproperlyConfigured

    conf, secrets = settings.load_conf(), settings.load_secrets()

    try:
        settings.USE_FLAT_FILES = conf["USE_FLAT_FILES"]
        settings.CLIENT_ID = secrets["CLIENT_ID"]
        settings.CLIENT_SECRET = secrets["CLIENT_SECRET"]
        settings.EMAIL_ADDRESS = secrets["EMAIL_ADDRESS"]
        settings.EMAIL_PASSWORD = secrets["EMAIL_PASSWORD"]
        settings.DB_USER = secrets["DB_USER"]
        settings.DB_PASSWORD = secrets["DB_PASSWORD"]
    except KeyError as e:
        if "USE_FLAT_FILES" in e.args[0]:
            raise ImproperlyConfigured(f'{e.args[0]} not found in conf.py')
        else:
            raise ImproperlyConfigured(f'{e.args[0]} not found in {conf["SECRETS_LOCATION"]}')
    return secrets


def __except__(exception, replacement_function):
    def _try_wrap(function):
        def __try_wrap(*__args, **__kwargs):
            try:
                return function(*__args, **__kwargs)
            except exception as e:
                return replacement_function(*__args, **__kwargs)
        return __try_wrap
    return _try_wrap


"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# To use a consistent encoding
from codecs import open
from os import path, listdir, getcwd

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

try:
    import pypandoc
    # Get the long description from the README file
    long_description = pypandoc.convert('README.rst', 'rst')
except(IOError, ImportError):
    long_description = open('README.rst').read()


setup(
    name='bullhorn_interface',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='1.0.0.dev',

    description='A simple Python package to facilitate interactions with the Bullhorn REST API',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/jjorissen52/bullhorn_interface',

    # Author details
    author='John-Paul Jorissen',
    author_email='jjorissen52@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.6',
        'Operating System :: Microsoft :: Windows :: Windows 10',
        'Operating System :: POSIX :: Linux',
        'Topic :: Office/Business',


        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.6',
    ],

    # What does your project relate to?
    keywords='Bullhorn Python API RESTAPI',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['some_files']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['psycopg2==2.7.1', 'SQLAlchemy==1.1.11', 'requests==2.18.1'],

    python_requires='>=3.6',
)